# pylint: disable=invalid-name, too-few-public-methods
# pylint: disable=too-many-instance-attributes

"""Eigen building block"""

from __future__ import absolute_import
from __future__ import unicode_literals
from __future__ import print_function

import os
import hpccm.templates.wget

from hpccm.building_blocks.base import bb_base
from hpccm.building_blocks.packages import packages
from hpccm.primitives.comment import comment
from hpccm.primitives.environment import environment
from hpccm.primitives.shell import shell

class eigen(bb_base, hpccm.templates.CMakeBuild, hpccm.templates.rm,
            hpccm.templates.tar, hpccm.templates.wget):
    """The `eigen` building block downloads and installs the
    [Eigen](http://eigen.tuxfamily.org) component.

    # Parameters

    prefix: The top level installation location.  The default value
    is `/usr/local/eigen`.

    version: The version of Eigen source to download.  The default
    value is `3.3.4`.

    # Examples

    ```python
    eigen(version='3.2.9')
    ```

    """
    def __init__(self, **kwargs):
        """Initialize building block"""
        super(eigen, self).__init__()

        self.__baseurl = kwargs.get('baseurl',
                                    'http://bitbucket.org/eigen/eigen/get')
        self.__ospackages = kwargs.get('ospackages', [])
        self.__parallel = kwargs.get('parallel', '$(nproc)')
        self.__prefix = kwargs.get('prefix', '/usr/local/eigen')
        self.__version = kwargs.get('version', '3.3.4')

        self.__commands = [] # Filled in by __setup()
        self.__environment_variables = {} # Filled in by __setup()
        self.__wd = '/var/tmp' # working directory

        # Construct the series of steps to execute
        self.__setup()

        self.__instructions

    def __instructions(self):
        """String representation of the building block"""
        self += comment('Eigen version {}'.format(self.__version))
        self += packages(ospackages=self.__ospackages)
        self += shell(commands=self.__commands)
        if self.__environment_variables:
            self += environment(variables=self.__environment_variables)


    def __setup(self):
        """Construct the series of shell commands, i.e., fill in
           self.__commands"""

        tarball = '{}.tar.gz'.format(self.__version)
        url = '{0}/{1}'.format(self.__baseurl, tarball)

        # Download source from web
        self.__commands.append(self.download_step(url=url,
                                                  directory=self.__wd))
        self.__commands.append(self.untar_step(
            tarball=os.path.join(self.__wd, tarball), directory=self.__wd))
        self.__commands.append('mv {0}/eigen-* {0}/eigen-{1}'.format(self.__wd, self.__version))

        self.__commands.extend([
            self.configure_step(
            directory='{0}/eigen-{1}'.format(self.__wd, self.__version),
            build_directory='{}/build'.format(self.__wd),
            opts=['-DCMAKE_INSTALL_PREFIX={0}'.format(self.__prefix)]),
        self.build_step(target='install', parallel=self.__parallel)
        ])

        # Set library path
        self.__environment_variables['Eigen3_ROOT'] = '{}'.format(self.__prefix)
        self.__environment_variables['Eigen3_DIR'] = '{}'.format(self.__prefix)

        # Cleanup tarball and directories
        self.__commands.append(self.cleanup_step(
            items=[os.path.join(self.__wd, tarball),
                   os.path.join(self.__wd, 'build'),
                   os.path.join(self.__wd,
                                'eigen-{}'.format(self.__version))]))

    # def runtime(self, _from='0'):
        # """Generate the set of instructions to install the runtime specific
        # components from a build in a previous stage.
        # """
        # instructions = []
        # instructions.append(comment('Eigen'))
        # instructions.append(copy(_from=_from, src=self.__prefix,
                                #  dest=self.__prefix))
#
        # if self.__environment_variables:
            # instructions.append(environment(
                # variables=self.__environment_variables))
        # return '\n'.join(str(x) for x in instructions)



# pylint: disable=invalid-name, too-few-public-methods
# pylint: disable=too-many-instance-attributes

"""Package manager spack building block"""

from __future__ import absolute_import
from __future__ import unicode_literals
from __future__ import print_function

from hpccm.building_blocks.packages import packages
from hpccm.primitives.comment import comment
from hpccm.primitives.environment import environment
from hpccm.primitives.label import label
from hpccm.primitives.shell import shell
from hpccm.templates.git import git


class pm_spack(object):
    """Package manager spack building block"""

    def __init__(self, **kwargs):
        """Initialize building block"""

        # Trouble getting MRO with kwargs working correctly, so just call
        # the parent class constructors manually for now.
        # super(python, self).__init__(**kwargs)

        self.__ospackages = kwargs.get('ospackages', [])

        self.__commands = []  # Filled in by __setup()
        self.__wd = '/var/tmp'  # working directory

        self.__setup()

    def __str__(self):
        """String representation of the building block"""
        instructions = [comment(__doc__, reformat=False),
                        packages(ospackages=self.__ospackages),
                        packages(yum=['xz'], apt=['xz-utils']),
                        environment(variables={
                            'PATH': '/opt/spack/bin:$PATH',
                            'FORCE_UNSAFE_CONFIGURE': '1'
                        }), shell(commands=self.__commands),
                        label(metadata={'org.opengeosys.pm': 'spack'})]
        # Without the FORCE_UNSAFE_CONFIGURE env var some spack package
        # installations may fail due to running as root.

        return '\n'.join(str(x) for x in instructions)

    def __setup(self):
        self.__ospackages.extend(['patch'])
        self.__commands.extend([
            git().clone_step(repository='https://github.com/spack/spack',
                             branch='develop', path='/opt'),
            'spack bootstrap',
            # TODO: There is no init system inside the container -> files are
            #       not sourced!
            'ln -s /opt/spack/share/spack/setup-env.sh /etc/profile.d/spack.sh',
            'ln -s /opt/spack/share/spack/spack-completion.bash /etc/profile.d',
            'spack clean --all'
        ])

    def runtime(self, _from='0'):
        """Install the runtime from a full build in a previous stage.  In this
           case there is no difference between the runtime and the
           full build."""
        return str(self)



#!/usr/bin/env python
import subprocess

subprocess.run(f"python build.py --pm system --cvode "
               "--ompi off 2.1.2 3.1.2 4.0.0",
               shell=True)

subprocess.run(f"python build.py --file Dockerfile.gcc.full --jenkins "
               "--cppcheck --docs --gcovr",
               shell=True)
subprocess.run(f"python build.py --file Dockerfile.gcc.gui --jenkins "
               "--cppcheck --gui --gcovr",
               shell=True)
subprocess.run(f"python build.py --file Dockerfile.clang.full "
               "--base_image ubuntu:18.04 --compiler 7 --clang --jenkins "
               "--iwyy",
               shell=True)



# pylint: disable=invalid-name, too-few-public-methods
# pylint: disable=too-many-instance-attributes

"""cppcheck building block"""

from __future__ import absolute_import
from __future__ import unicode_literals
from __future__ import print_function

import os
import hpccm.templates.wget

from hpccm.building_blocks.base import bb_base
from hpccm.building_blocks.packages import packages
from hpccm.primitives.comment import comment
from hpccm.primitives.environment import environment
from hpccm.primitives.shell import shell


class cppcheck(bb_base, hpccm.templates.CMakeBuild, hpccm.templates.rm,
               hpccm.templates.tar, hpccm.templates.wget):
    """The `cvode` building block downloads and installs the
    [cppcheck](https://computation.llnl.gov/projects/sundials/cvode) component.

    # Parameters

    prefix: The top level installation location.  The default value
    is `/usr/local/cppcheck`.

    version: The version of cppcheck source to download.  The default
    value is `1.83`.
    """

    def __init__(self, **kwargs):
        """Initialize building block"""
        super(cppcheck, self).__init__()

        self.__baseurl = kwargs.get('baseurl',
                                    'https://github.com/danmar/cppcheck/archive')
        self.__ospackages = kwargs.get('ospackages', [])
        self.__parallel = kwargs.get('parallel', '$(nproc)')
        self.__prefix = kwargs.get('prefix', '/usr/local/cppcheck')
        self.__version = kwargs.get('version', '1.87')

        self.__commands = [] # Filled in by __setup()
        self.__environment_variables = {} # Filled in by __setup()
        self.__wd = '/var/tmp' # working directory

        # Construct the series of steps to execute
        self.__setup()

        self.__instructions()

    def __instructions(self):
        self += comment('cppcheck version {}'.format(self.__version))
        self += packages(ospackages=self.__ospackages)
        self += shell(commands=self.__commands)
        if self.__environment_variables:
            self += environment(variables=self.__environment_variables)

    def __setup(self):
        """Construct the series of shell commands, i.e., fill in
           self.__commands"""

        tarball = '{}.tar.gz'.format(self.__version)
        url = '{0}/{1}'.format(self.__baseurl, tarball)

        # Download source from web
        self.__commands.append(self.download_step(url=url,
                                                  directory=self.__wd))
        self.__commands.append(self.untar_step(
            tarball=os.path.join(self.__wd, tarball), directory=self.__wd))

        self.__commands.extend([
            self.configure_step(
            directory='{0}/cppcheck-{1}'.format(self.__wd, self.__version),
            build_directory='{}/build'.format(self.__wd),
            opts=['-DCMAKE_INSTALL_PREFIX={0}'.format(self.__prefix),
                '-DCMAKE_BUILD_TYPE=RELEASE']),
        self.build_step(target='install', parallel=self.__parallel)
        ])

        self.__environment_variables['PATH'] = '{0}/bin:$PATH'.format(self.__prefix)

        # Cleanup tarball and directories
        self.__commands.append(self.cleanup_step(
            items=[os.path.join(self.__wd, tarball),
                   os.path.join(self.__wd, 'build'),
                   os.path.join(self.__wd,
                                'cppcheck-{}'.format(self.__version))]))

    # No runtime



# pylint: disable=invalid-name, too-few-public-methods
# pylint: disable=too-many-instance-attributes

"""PETSc building block"""

from __future__ import absolute_import
from __future__ import unicode_literals
from __future__ import print_function

import os
import hpccm.templates.wget

from hpccm.building_blocks.base import bb_base
from hpccm.building_blocks.packages import packages
from hpccm.primitives.comment import comment
from hpccm.primitives.copy import copy
from hpccm.primitives.environment import environment
from hpccm.primitives.label import label
from hpccm.primitives.shell import shell
from hpccm.toolchain import toolchain


class petsc(bb_base, hpccm.templates.ConfigureMake, hpccm.templates.rm,
            hpccm.templates.tar, hpccm.templates.wget):
    """The `PETSc` building block downloads and installs the
    [VTK](https://vtk.org/) component.

    # Parameters

    prefix: The top level installation location.  The default value
    is `/usr/local/petsc`.

    version: The version of PETSc source to download.  The default
    value is `3.8.4`.

    # Examples

    ```python
    petsc(version='3.8.1')
    ```

    """
    def __init__(self, **kwargs):
        super(petsc, self).__init__()

        self.__ospackages = kwargs.get('ospackages', [])
        self.parallel = 1
        self.prefix = kwargs.get('prefix', '/usr/local/petsc')
        self.__toolchain = toolchain(CC='mpicc', CXX='mpicxx')
        self.configure_opts = kwargs.get('configure_opts', [])
        self.__version = kwargs.get('version', '3.8.4')
        self.__wd = '/var/tmp' # working directory
        self.__baseurl = kwargs.get('baseurl', 'http://ftp.mcs.anl.gov/pub/petsc/release-snapshots')

        # Filled in by __setup():
        self.__commands = []
        self.__environment_variables = {}
        self.__labels = {}

        self.__setup()

        self.__instructions()

    def __instructions(self):
        self += comment('PETSc {}'.format(self.__version))
        self += packages(ospackages=self.__ospackages)
        self += shell(commands=self.__commands)
        if self.__environment_variables:
            self += environment(variables=self.__environment_variables)
        if self.__labels:
            self += label(metadata=self.__labels)


    def __setup(self):
        """Construct the series of shell commands, i.e., fill in
           self.__commands"""

        # Get the source
        directory = 'petsc-{}'.format(self.__version)
        tarball = 'petsc-lite-{}.tar.gz'.format(self.__version)
        url = '{0}/{1}'.format(self.__baseurl, tarball)

        self.__commands.append(self.download_step(url=url,
                                                  directory=self.__wd))
        self.__commands.append(self.untar_step(
            tarball=os.path.join(self.__wd, tarball), directory=self.__wd))

        # Default configure opts
        self.configure_opts.extend([
            'CC={}'.format(self.__toolchain.CC),
            'CXX={}'.format(self.__toolchain.CXX),
            '--CFLAGS=\'-O3\'',
            '--CXXFLAGS=\'-O3\'',
            '--FFLAGS=\'-O3\'',
            '--with-debugging=no',
            '--with-fc=0',
            '--download-f2cblaslapack=1'
        ])

        # Configure, build, install
        self.__commands.append(self.configure_step(
            directory=os.path.join(self.__wd, directory)))
        self.__commands.append(self.build_step())
        self.__commands.append(self.install_step())

        # Cleanup tarball and directory
        self.__commands.append(self.cleanup_step(
            items=[os.path.join(self.__wd, tarball),
                   os.path.join(self.__wd, directory)]))

        # Environment
        self.__environment_variables['PETSC_DIR'] = '{}'.format(self.prefix)
        libpath = os.path.join(self.prefix, 'lib')
        self.__environment_variables['LD_LIBRARY_PATH'] = '{}:$LD_LIBRARY_PATH'.format(libpath)

        # Labels
        self.__labels['petsc.version'] = self.__version

    def runtime(self, _from='0'):
        instructions = []
        instructions.append(comment('PETSc {}'.format(self.__version)))
        instructions.append(copy(_from=_from, src=self.prefix,
                                 dest=self.prefix))

        if self.__environment_variables:
            instructions.append(environment(
                variables=self.__environment_variables))
        if self.__labels:
            instructions.append(label(metadata=self.__labels))
        return '\n'.join(str(x) for x in instructions)



# pylint: disable=invalid-name, too-few-public-methods
# pylint: disable=too-many-instance-attributes

"""VTK building block"""

from __future__ import absolute_import
from __future__ import unicode_literals
from __future__ import print_function

import re
import os
import hpccm.templates.wget

from hpccm.building_blocks.base import bb_base
from hpccm.building_blocks.packages import packages
from hpccm.primitives.comment import comment
from hpccm.primitives.copy import copy
from hpccm.primitives.environment import environment
from hpccm.primitives.shell import shell
from hpccm.toolchain import toolchain


class vtk(bb_base, hpccm.templates.CMakeBuild, hpccm.templates.rm,
          hpccm.templates.tar, hpccm.templates.wget):
    """The `VTK` building block downloads and installs the
    [VTK](https://vtk.org/) component.

    # Parameters

    prefix: The top level installation location.  The default value
    is `/usr/local/vtk`.

    version: The version of VTK source to download.  The default
    value is `8.2.0`.

    # Examples

    ```python
    vtk(version='7.1.1')
    ```

    """
    def __init__(self, **kwargs):
        super(vtk, self).__init__()

        self.__cmake_args = kwargs.get('cmake_args', [])
        self.__ospackages = kwargs.get('ospackages', [])
        self.__parallel = kwargs.get('parallel', '$(nproc)')
        self.__prefix = kwargs.get('prefix', '/usr/local/vtk')
        self.__shared = kwargs.get('shared', True)
        self.__toolchain = kwargs.get('toolchain', toolchain())
        self.__version = kwargs.get('version', '8.2.0')
        self.__wd = '/var/tmp' # working directory
        match = re.match(r'(?P<major>\d+)\.(?P<minor>\d+)\.(?P<revision>\d+)',
                         self.__version)
        short_version = '{0}.{1}'.format(match.groupdict()['major'], match.groupdict()['minor'])
        self.__baseurl = kwargs.get('baseurl',
                                    'https://www.vtk.org/files/release/{0}'.format(short_version))

        # Filled in by __setup():
        self.__commands = []
        self.__environment_variables = {}

        self.__setup()

        self.__instructions()

    def __instructions(self):
        self += comment('VTK {}'.format(self.__version))
        self += packages(ospackages=self.__ospackages)
        self += shell(commands=self.__commands)
        if self.__environment_variables:
            self += environment(variables=self.__environment_variables)


    def __setup(self):
        """Construct the series of shell commands, i.e., fill in
           self.__commands"""

        tarball = 'VTK-{}.tar.gz'.format(self.__version)
        url = '{0}/{1}'.format(self.__baseurl, tarball)

        # Default CMake arguments
        self.__cmake_args.extend([
          '-DCMAKE_INSTALL_PREFIX={0}'.format(self.__prefix),
          '-DCMAKE_BUILD_TYPE=Release'
        ])
        if not self.__shared:
            self.__cmake_args.append('-DBUILD_SHARED_LIBS=OFF')
        if self.__toolchain.CC == 'mpicc':
            self.__cmake_args.extend([
                '-DModule_vtkIOParallelXML=ON',
                '-DModule_vtkParallelMPI=ON'])

        # Download source from web
        self.__commands.append(self.download_step(url=url,
                                                  directory=self.__wd))
        self.__commands.append(self.untar_step(
            tarball=os.path.join(self.__wd, tarball), directory=self.__wd))

        # Configure and build
        self.__commands.extend([
            self.configure_step(
              directory='{0}/VTK-{1}'.format(self.__wd, self.__version),
              build_directory='{}/build'.format(self.__wd),
              opts=self.__cmake_args,
              toolchain=self.__toolchain),
            self.build_step(target='install', parallel=self.__parallel)
        ])

        # Set library path
        self.__environment_variables['VTK_ROOT'] = '{}'.format(self.__prefix)
        libpath = os.path.join(self.__prefix, 'lib')
        if self.__shared:
            self.__environment_variables['LD_LIBRARY_PATH'] = '{}:$LD_LIBRARY_PATH'.format(libpath)

        # Cleanup tarball and directory
        self.__commands.append(self.cleanup_step(
            items=[os.path.join(self.__wd, tarball),
                   os.path.join(self.__wd, 'build'),
                   os.path.join(self.__wd,
                                'VTK-{}'.format(self.__version))]))

    def runtime(self, _from='0'):
        if not self.__shared:
            return str(comment('VTK (empty)'))
        instructions = []
        instructions.append(comment('VTK {}'.format(self.__version)))
        instructions.append(copy(_from=_from, src=self.__prefix,
                                 dest=self.__prefix))

        if self.__environment_variables:
            instructions.append(environment(
                variables=self.__environment_variables))
        return '\n'.join(str(x) for x in instructions)



#!/usr/bin/env python3

# OpenMPI versions:
#  Taurus: 1.8.8, 1.10.2, 2.1.0, 2.1.1, 3.0.0, 3.1.2
#  Eve: 1.8.8, 1.10.2, 2.1.1, 4.0.0
#  --> 2.1.1
# https://easybuild.readthedocs.io/en/latest/Common-toolchains.html#common-toolchains-overview
# easybuild toolchain: 2017b (2.1.1), 2018a (2.1.2), 2018b (3.1.1)
import argparse
import hashlib
import itertools
import json
import math
import multiprocessing
import os
import requests
import subprocess
import sys

import hpccm
from hpccm import linux_distro
from hpccm.building_blocks import packages, mlnx_ofed, knem, ucx, openmpi, \
    boost, pip, scif, llvm, gnu
from hpccm.primitives import comment, user, environment, raw, label

import ogscm
from ogscm.config import package_manager
from ogscm.version import __version__
from ogscm.building_blocks import ccache, cppcheck, cvode, eigen, iwyy, \
    jenkins_node, ogs_base, ogs, osu_benchmarks, petsc, pm_conan, vtk

def main(): # pragma: no cover
    cli = argparse.ArgumentParser(
        description='Generate container image definitions.',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    cli.add_argument('--recipe', type=str, default='recipes/ogs-builder.py',
                     help='HPCCM recipe')
    cli.add_argument('--out', type=str, default='_out', help='Output directory')
    cli.add_argument('--file', type=str, default='', help='Overwrite output recipe file name')
    cli.add_argument('--print', '-P', dest='print', action='store_true',
                     help='Print the definition to stdout')
    options_g = cli.add_argument_group('Combinatorial options',
                                       'All combinations of the given options will '
                                       'be generated')
    options_g.add_argument('--format', nargs='*', type=str,
                           choices=['docker', 'singularity'],
                           default=['docker'])
    options_g.add_argument('--pm', nargs='*', type=str,
                           choices=['system', 'conan'], default=['conan'],
                           help='Package manager to install third-party '
                                'dependencies')
    options_g.add_argument('--ompi', nargs='*', type=str,
                           default=['off'],
                           help='OpenMPI version, e.g. 2.1.1, 2.1.5, 3.0.1, 3.1.2')
    options_g.add_argument('--ogs', nargs='*', type=str,
                           default=['ufz/ogs@master'],
                           help='OGS GitHub repo in the form \'user/repo@branch\' '
                                'or \'off\' to disable OGS building')
    options_g.add_argument('--cmake_args', nargs='*', type=str, default=[''],
                           help='CMake argument sets have to be quoted and **must**'
                                ' start with a space. e.g. --cmake_args \' -DFIRST='
                                'TRUE -DFOO=BAR\' \' -DSECOND=TRUE\'')
    build_g = cli.add_argument_group('Image build options')
    build_g.add_argument('--build', '-B', dest='build', action='store_true',
                         help='Build the images from the definition files')
    build_g.add_argument('--upload', '-U', dest='upload', action='store_true',
                         help='Upload Docker image to registry')
    build_g.add_argument('--registry', type=str,
                         default='registry.opengeosys.org/ogs/ogs',
                         help='The docker registry the image is tagged and '
                              'uploaded to.')
    build_g.add_argument('--convert', '-C', dest='convert', action='store_true',
                         help='Convert Docker image to Singularity image')
    build_g.add_argument('--runtime-only', '-R', dest='runtime_only',
                         action='store_true',
                         help='Generate multi-stage Dockerfiles for small runtime '
                              'images')
    switches_g = cli.add_argument_group('Additional options')
    switches_g.add_argument('--base_image', type=str, default='ubuntu:17.10',
                            help='The base image. \'centos:7\' is supported too.')
    switches_g.add_argument('--clang', dest='clang', action='store_true',
                            help='Use clang instead of gcc')
    switches_g.add_argument('--gui', dest='gui', action='store_true',
                            help='Builds the GUI (Data Explorer)')
    switches_g.add_argument('--docs', dest='docs', action='store_true',
                            help='Setup documentation requirements (Doxygen)')
    switches_g.add_argument('--jenkins', dest='jenkins', action='store_true',
                            help='Setup Jenkins slave')
    switches_g.add_argument('--cvode', dest='cvode', action='store_true',
                            help='Install and configure with cvode')
    switches_g.add_argument('--cppcheck', dest='cppcheck', action='store_true',
                            help='Install cppcheck')
    switches_g.add_argument('--iwyy', dest='iwyy', action='store_true',
                            help='Install include-what-you-use')
    switches_g.add_argument('--gcovr', dest='gcovr', action='store_true',
                            help='Install gcovr')
    switches_g.add_argument('--dev', dest='dev', action='store_true',
                            help='Installs development tools (vim, gdb)')
    switches_g.add_argument('--compiler_version', type=str, default='',
                            help='Compiler version.')
    cli.set_defaults(build=False)
    cli.set_defaults(convert=False)
    cli.set_defaults(print=False)
    cli.set_defaults(runtime_only=False)
    cli.set_defaults(upload=False)
    cli.set_defaults(clang=False)
    cli.set_defaults(gui=False)
    cli.set_defaults(docs=False)
    cli.set_defaults(jenkins=False)
    cli.set_defaults(cvode=False)
    cli.set_defaults(cppcheck=False)
    cli.set_defaults(iwyy=False)
    cli.set_defaults(gcovr=False)
    cli.set_defaults(dev=False)
    args = cli.parse_args()

    # if not len(sys.argv) > 1:
    #     cli.print_help()
    #     exit(0)

    if args.jenkins:
        args.ogs = ['off']

    c = list(itertools.product(args.format, args.ogs, args.pm, args.ompi,
                               args.cmake_args))
    if not args.print:
        print('Creating {} image definition(s)...'.format(len(c)))
    for build in c:
        __format = build[0]
        ogs_version = build[1]
        ogscm.config.set_package_manager(build[2])
        ompi = build[3]
        cmake_args = build[4].strip().split(' ')

        # args checking
        if len(c) > 1 and args.file != '':
            print('--file can only be used when generating a single image definition')
            quit(1)
        if ogs_version == 'off' and len(cmake_args) > 0 and cmake_args[0] != '':
            cmake_args = []
            print('--cmake_args cannot be used with --ogs off! Ignoring!')
        if __format == 'singularity':
            if args.runtime_only:
                args.runtime_only = False
                print('--runtime-only cannot be used with --format singularity! '
                      'Ignoring!')
            if args.upload:
                print('--upload cannot be used with --format singularity! '
                      'Ignoring!')
            if args.convert:
                print('--convert cannot be used with --format singularity! '
                      'Ignoring!')

        if len(cmake_args) > 0:
            cmake_args_hash = hashlib.md5(
                ' '.join(cmake_args).encode('utf-8')).hexdigest()
            cmake_args_hash_short = cmake_args_hash[:8]

        commit_hash = '0'
        ogs_tag = ''

        name_image = args.base_image.replace(':', '_')
        name_start = 'gcc'
        if ogs_version != 'off':
            # Get git commit hash and construct image tag name
            repo, branch = ogs_version.split("@")
            url = f"https://api.github.com/repos/{repo}/commits?sha={branch}"
            response = requests.get(url)
            response_data = json.loads(response.text)
            commit_hash = response_data[0]['sha']
            ogs_tag = ogs_version.replace('/', '.').replace('@', '.')
            name_start = f'ogs-{commit_hash[:8]}'
        elif args.clang:
            name_start = 'clang'

        name_openmpi = 'serial'
        if ompi != 'off':
            name_openmpi = f"openmpi-{ompi}"

        img_file = f"{name_image}-{name_start}-{name_openmpi}-{ogscm.config.g_package_manager.name.lower()}"
        img_folder = f"{name_image}/{name_start}/{name_openmpi}/{ogscm.config.g_package_manager.name.lower()}"
        if len(cmake_args) > 0:
            img_file += f'-cmake-{cmake_args_hash_short}'
        if args.gui:
            img_file += '-gui'
        if args.docs:
            img_file += '-docs'
        if ogs_version != 'off' and not args.runtime_only:
            img_file += '-dev'
        docker_repo = img_file
        img_file += '.sif'
        if __format == 'singularity':
            subprocess.run(f"sudo `which singularity` build {images_out_dir}/{img_file} "
                f"{definition_file}", shell=True)
            subprocess.run(f"sudo chown $USER:$USER {images_out_dir}/{img_file}", shell=True)
            continue

        tag = f"{args.registry}/{docker_repo}"

        # paths
        if args.file != '':
            out_dir = args.out
            definition_file = args.file
        else:
            out_dir = f"{args.out}/{__format}/{img_folder}"
            if len(cmake_args) > 0:
                out_dir += f'/cmake-{cmake_args_hash_short}'
            images_out_dir = "_out/images"
            if not os.path.exists(out_dir):
                os.makedirs(out_dir)
            if not os.path.exists(images_out_dir):
                os.makedirs(images_out_dir)
            definition_file = 'Dockerfile'
            if __format == 'singularity':
                definition_file = 'Singularity.def'
            # definition_file = os.path.join(out_dir, definition_file)

        # Create definition
        base_cwd = os.getcwd()
        recipe_file = os.path.join(base_cwd, args.recipe)
        os.chdir(out_dir)

        hpccm.config.set_container_format(__format)

        # ------------------------------ recipe ------------------------------------
        Stage0 = hpccm.Stage()
        if args.runtime_only:
            Stage0.name = 'stage0'
        Stage0.baseimage(image=args.base_image)
        centos = hpccm.config.g_linux_distro == linux_distro.CENTOS

        # Get git info
        # local_git_hash = subprocess.check_output([
        #     'git', 'rev-parse', '--short', 'HEAD']).decode('ascii').strip()
        Stage0 += comment(
            # "Generated with https://github.com/ufz/ogs-container-maker/commit/{0}".format(
            #     local_git_hash),
            f"Generated with ogs-container-maker {__version__}",
            reformat=False)

        if centos:
            Stage0 += user(user='root')
            Stage0 += packages(ospackages=['epel-release'])
        Stage0 += packages(ospackages=['wget', 'tar', 'curl'])

        # base compiler
        if args.compiler_version == '':
            if args.clang:
                if hpccm.config.g_linux_distro == linux_distro.CENTOS:
                    # installs llvm-toolset-7-clang which is clang 5.0.1
                    args.compiler_version = '7'
                else:
                    args.compiler_version = '5.0'
            else:
                args.compiler_version = '6'
        if args.clang:
            compiler = llvm(extra_repository=True, version=args.compiler_version)
        else:
            compiler = gnu(fortran=False, extra_repository=True,
                           version=args.compiler_version)
        toolchain = compiler.toolchain
        Stage0 += compiler
        if args.clang:
            Stage0 += packages(
                apt=["clang-tidy-{}".format(args.compiler_version),
                     "clang-format-{}".format(args.compiler_version)],
                yum=["llvm-toolset-{}-clang-tools-extra".format(args.compiler_version)]
            )

        if ompi != 'off':
            Stage0 += mlnx_ofed()  # version='3.4-1.0.0.0'
            Stage0 += knem()
            Stage0 += ucx(cuda=False, knem='/usr/local/knem')

            mpicc = openmpi(version=ompi, cuda=False, ucx='/usr/local/ucx',
                            configure_opts=[
                                '--with-slurm',
                                '--enable-mca-no-build=btl-openib,plm-slurm'
                            ],
                            # ospackages=['libslurm-dev'],
                            toolchain=toolchain)
            toolchain = mpicc.toolchain
            Stage0 += mpicc

            Stage0 += label(metadata={
                'org.opengeosys.mpi': 'openmpi',
                'org.opengeosys.mpi.version': ompi
            })

            if True:  # TODO configurable?
                Stage0 += osu_benchmarks()

        Stage0 += ogs_base()
        if args.gui:
            Stage0 += packages(ospackages=[
                'mesa-common-dev', 'libgl1-mesa-dev', 'libxt-dev'
            ])
        if ogscm.config.g_package_manager == package_manager.CONAN:
            Stage0 += pm_conan()
            if not args.jenkins:
                Stage0 += environment(variables={'CONAN_SYSREQUIRES_SUDO': 0})
        elif package_manager.get() == package_manager.SYSTEM:
            Stage0 += boost()  # header only?
            Stage0 += environment(variables={'BOOST_ROOT': '/usr/local/boost'})
            Stage0 += eigen()
            vtk_cmake_args = [
                '-DVTK_Group_StandAlone=OFF',
                '-DVTK_Group_Rendering=OFF',
                '-DModule_vtkIOXML=ON'
            ]
            Stage0 += vtk(cmake_args=vtk_cmake_args, toolchain=toolchain)
            if ompi != 'off':
                Stage0 += petsc()
        if args.cvode:
            Stage0 += cvode()
        if args.cppcheck:
            Stage0 += cppcheck()
        if args.iwyy and args.clang:
            Stage0 += iwyy(clang_version=args.compiler_version)
        if args.docs:
            Stage0 += packages(
                ospackages=['doxygen', 'graphviz', 'texlive-base'])
        if args.gcovr:
            Stage0 += pip(pip='pip3', packages=['gcovr'])

        if args.dev:
            Stage0 += packages(ospackages=['vim', 'gdb'])

        if ogs_version != 'off':
            if args.cvode:
                cmake_args.append('-DOGS_USE_CVODE=ON')
            if args.gui:
                cmake_args.append('-DOGS_BUILD_GUI=ON')

            Stage0 += pip(packages=['scif'])  # SCI-F
            Stage0 += raw(docker='ARG OGS_COMMIT_HASH=0')
            ogs_app = scif(name='ogs')
            ogs_app += ogs(version=ogs_version, toolchain=toolchain,
                           prefix='/scif/apps/ogs',
                           cmake_args=cmake_args,
                           parallel=math.ceil(multiprocessing.cpu_count() / 2),
                           skip_lfs=True, remove_build=True,
                           remove_source=True)  # TODO: maybe only in runtime image?
            Stage0 += ogs_app

        if args.jenkins:
            Stage0 += ccache(cache_size='15G')
            Stage0 += packages(
                ospackages=['sudo'])  # For user switching back to root
            Stage0 += jenkins_node()

        stages_string = str(Stage0)

        if args.runtime_only:
            Stage1 = hpccm.Stage()
            Stage1.baseimage(image=args.base_image)
            Stage1 += Stage0.runtime()
            Stage1 += pip(packages=['scif'])  # Install scif in runtime too
            stages_string += "\n\n" + str(Stage1)

        # ---------------------------- recipe end ----------------------------------

        with open(definition_file, 'w') as f:
            print(stages_string, file=f)
        if args.print:
            print(stages_string)
        else:
            print(f'Created definition {out_dir}/{definition_file}')

        # Create image
        if not args.build:
            continue

        build_cmd = (f"docker build --build-arg OGS_COMMIT_HASH={commit_hash} "
                     f"-t {tag} .")
        print(f"Running: {build_cmd}")
        subprocess.run(build_cmd, shell=True)
        if args.upload:
            subprocess.run(f"docker push {tag}", shell=True)
        if args.convert:
            subprocess.run(f"docker run -d -p 5000:5000 --rm  --name registry registry:2 && "
                f"docker tag {tag} localhost:5000/{tag} && "
                f"docker push localhost:5000/{tag} && "
                f"SINGULARITY_NOHTTPS=true singularity build --force {images_out_dir}/{img_file} docker://localhost:5000/{tag} && "
                f"docker rmi localhost:5000/{tag} && "
                f"docker stop registry",
                shell=True)

if __name__ == "__main__": # pragma: no cover
    main()



# pylint: disable=invalid-name, too-few-public-methods
# pylint: disable=too-many-instance-attributes

"""Include-what-you-use building block"""

from __future__ import absolute_import
from __future__ import unicode_literals
from __future__ import print_function

import os
import hpccm.templates.wget

from hpccm.building_blocks.base import bb_base
from hpccm.building_blocks.packages import packages
from hpccm.primitives.comment import comment
from hpccm.primitives.environment import environment
from hpccm.primitives.shell import shell

class iwyy(bb_base, hpccm.templates.CMakeBuild, hpccm.templates.rm,
           hpccm.templates.tar, hpccm.templates.wget):
    """
    # Parameters

    prefix: The top level installation location.  The default value
    is `/usr/local/iwyy`.

    clang_version: The installed clang version. Required!

    # Examples

    ```python
    iwyy(clang_version='5.0')
    ```

    """
    def __init__(self, **kwargs):
        """Initialize building block"""
        super(iwyy, self).__init__()

        self.__ospackages = kwargs.get('ospackages', [])
        self.__parallel = kwargs.get('parallel', '$(nproc)')
        self.__prefix = kwargs.get('prefix', '/usr/local/iwyy')
        self.__clang_version = kwargs.get('clang_version')

        self.__commands = []  # Filled in by __setup()
        self.__environment_variables = {}  # Filled in by __setup()
        self.__wd = '/var/tmp'  # working directory

        # Construct the series of steps to execute
        self.__setup()

        self.__instructions()

    def __instructions(self):
        """String representation of the building block"""


        self += comment('Include-what-you-use for clang version {}'.format(
                        self.__clang_version))
        self += packages(ospackages=self.__ospackages)
        self += shell(commands=self.__commands)
        if self.__environment_variables:
            self += environment(variables=self.__environment_variables)


    def __setup(self):
        """Construct the series of shell commands, i.e., fill in
           self.__commands"""

        self.__ospackages.extend([
            'libncurses5-dev',
            'zlib1g-dev',
            'llvm-{}-dev'.format(self.__clang_version),
            'libclang-{}-dev'.format(self.__clang_version)
        ])

        baseurl = "https://github.com/include-what-you-use/include-what-you-use/archive"
        tarball = 'clang_{}.0.tar.gz'.format(self.__clang_version)
        url = '{0}/{1}'.format(baseurl, tarball)
        directory = "include-what-you-use-clang_{0}.0".format(self.__clang_version)

        # Download source from web
        self.__commands.append(self.download_step(url=url,
                                                  directory=self.__wd))
        self.__commands.append(self.untar_step(
            tarball=os.path.join(self.__wd, tarball), directory=self.__wd))

        self.__commands.extend([
            self.configure_step(
                directory='{0}/{1}'.format(self.__wd, directory),
                build_directory='{}/build'.format(self.__wd),
                opts=[
                    '-DCMAKE_INSTALL_PREFIX={}'.format(self.__prefix),
                    '-DIWYU_LLVM_ROOT_PATH=/usr/lib/llvm-{}'.format(
                        self.__clang_version)]),
            self.build_step(target='install', parallel=self.__parallel)
        ])

        self.__environment_variables['PATH'] = '{}/bin:$PATH'.format(self.__prefix)

        # Cleanup tarball and directories
        self.__commands.append(self.cleanup_step(
            items=[os.path.join(self.__wd, tarball),
                   os.path.join(self.__wd, 'build'),
                   os.path.join(self.__wd, directory)]))

        # cleanup packages?
        # self.__commands.append(
        #     'apt remove -y {}'.format(' '.join(self.__ospackages))
        # )

    # no runtime



#!/usr/bin/env python
import argparse
import subprocess

cli = argparse.ArgumentParser()
cli.add_argument("--out", type=str, default="_out")
args = cli.parse_args()

subprocess.run(f"python build.py --out {args.out} --file Dockerfile.gcc.full "
               "--jenkins --cppcheck --docs --gcovr",
               shell=True)
subprocess.run(f"python build.py --out {args.out} --file Dockerfile.gcc.gui "
               "--jenkins --cppcheck --gui --gcovr",
               shell=True)
subprocess.run(f"python build.py --out {args.out} --file Dockerfile.clang.full "
               "--base_image ubuntu:18.04 --compiler 7 --clang --jenkins "
               "--iwyy",
               shell=True)



# pylint: disable=invalid-name, too-few-public-methods
# pylint: disable=too-many-instance-attributes
"""Package manager Conan building block"""

from __future__ import absolute_import
from __future__ import unicode_literals
from __future__ import print_function

from hpccm.building_blocks.base import bb_base
from hpccm.building_blocks.packages import packages
from hpccm.primitives.comment import comment
from hpccm.primitives.environment import environment
from hpccm.primitives.label import label
from hpccm.primitives.shell import shell


class ccache(bb_base):
    """Package manager Conan building block"""

    def __init__(self, **kwargs):
        """Initialize building block"""
        super(ccache, self).__init__()
        self.__cache_dir = kwargs.get('cache_dir', '/opt/cache')
        self.__cache_size = kwargs.get('cache_size', '5G')

        self.__instructions()

    def __instructions(self):
        self += comment(__doc__, reformat=False)
        self += packages(ospackages=['ccache'])
        self += shell(commands=[
            'mkdir -p {0} && chmod 777 {0}'.format(self.__cache_dir)
        ])
        self += environment(variables={
            'CCACHE_DIR': self.__cache_dir,
            'CCACHE_MAXSIZE': self.__cache_size,
            'CCACHE_SLOPPINESS': 'pch_defines,time_macros'
        })
        self += label(metadata={
            'ccache.dir': self.__cache_dir,
            'ccache.size': self.__cache_size
        })
    # No runtime



# pylint: disable=invalid-name, too-few-public-methods
# pylint: disable=too-many-instance-attributes

"""OGS building block"""

from __future__ import absolute_import
from __future__ import unicode_literals
from __future__ import print_function

import os
import re

import hpccm.templates.rm

from hpccm.building_blocks.base import bb_base
from hpccm.building_blocks.packages import packages
from hpccm.primitives.comment import comment
from hpccm.primitives.copy import copy
from hpccm.primitives.environment import environment
from hpccm.primitives.label import label
from hpccm.primitives.runscript import runscript
from hpccm.primitives.shell import shell
from hpccm.toolchain import toolchain

import ogscm
from ogscm.config import package_manager


class ogs(bb_base, hpccm.templates.CMakeBuild, hpccm.templates.rm):
    """OGS building block"""

    def __init__(self, **kwargs):
        """Initialize building block"""
        super(ogs, self).__init__(**kwargs)

        self.__cmake_args = kwargs.get('cmake_args', [])
        self.__ospackages = []
        self.__parallel = kwargs.get('parallel', 4)
        self.__prefix = kwargs.get('prefix', '/usr/local/ogs')
        self.__remove_dev = kwargs.get('remove_dev', False)
        self.__remove_build = kwargs.get('remove_build', False)
        self.__remove_source = kwargs.get('remove_source', False)
        self.__shared = kwargs.get('shared', True)
        self.__skip_lfs = kwargs.get('skip_lfs', False)
        self.__toolchain = kwargs.get('toolchain', toolchain())
        self.__version = kwargs.get('version', 'ufz/ogs@master')
        m = re.search('(.+/.*)@(.*)', self.__version)
        self.__repo = m.group(1)
        self.__branch = m.group(2)

        # Filled in by __setup():
        self.__commands = []
        self.__environment_variables = {}
        self.__labels = {}

        self.__setup()

        # Fill in container instructions
        self.__instructions()

    def __instructions(self):
        self += comment('OpenGeoSys build from repo {0}, branch {1}'.format(
                        self.__repo, self.__branch))
        self += packages(ospackages=self.__ospackages)
        self += shell(commands=self.__commands)
        self += runscript(commands=['ogs'])

        if self.__environment_variables:
            self += environment(variables=self.__environment_variables)
        if self.__labels:
            self += label(metadata=self.__labels)


    def __setup(self):
        """Construct the series of shell commands, i.e., fill in
           self.__commands"""
        conan = ogscm.config.g_package_manager == package_manager.CONAN

        # Get the source
        self.__commands.extend([
            'mkdir -p {0} && cd {0}'.format(self.__prefix),
            # TODO: --depth=1 --> ogs --version does not work
            '{}git clone --branch {} https://github.com/{} src'.format(
                'GIT_LFS_SKIP_SMUDGE=1 ' if self.__skip_lfs else '',
                self.__branch, self.__repo),
            "(cd src && git fetch --tags)"
        ])

        # Default CMake arguments
        self.__cmake_args.extend([
            "-G Ninja",
            "-DCMAKE_INSTALL_PREFIX={}".format(self.__prefix),
            "-DCMAKE_BUILD_TYPE=Release",
        ])

        self.__cmake_args.append('-DBUILD_SHARED_LIBS={}'.format(
            'ON' if self.__shared else 'OFF'
        ))
        if self.__skip_lfs:
            self.__cmake_args.append('-DBUILD_TESTING=OFF')
        if self.__toolchain.CC == 'mpicc':
            self.__cmake_args.append("-DOGS_USE_PETSC=ON")
            if conan == True:
                self.__cmake_args.append("-DOGS_CONAN_USE_SYSTEM_OPENMPI=ON")
        if conan == False:
            self.__cmake_args.append('-DOGS_USE_CONAN=OFF')

        # Configure and build
        self.__commands.append(self.configure_step(
            directory='{}/src'.format(self.__prefix),
            build_directory='{}/build'.format(self.__prefix),
            opts=self.__cmake_args,
            toolchain=self.__toolchain))
        self.__commands.append(self.build_step(
            target='install', parallel=self.__parallel))

        # Cleanup
        if self.__remove_build:
            # Remove whole src and build directories
            self.__commands.append(self.cleanup_step(
                items=[os.path.join(self.__prefix, 'build')]
            ))
        else:
            # Just run the clean-target
            self.__commands.append(self.build_step(target='clean'))
        if self.__remove_source:
            # Remove whole src and build directories
            self.__commands.append(self.cleanup_step(
                items=[os.path.join(self.__prefix, 'src')]
            ))

        # Environment
        self.__environment_variables['PATH'] = '{0}/bin:$PATH'.format(
            self.__prefix)

        # Labels
        self.__labels['version'] = self.__version
        self.__labels['cmake_args'] = '\'' + ' '.join(self.__cmake_args) + '\''

    def runtime(self, _from='0'):
        instructions = [
            comment('OpenGeoSys build from repo {0}, branch {1}'.format(
                self.__repo, self.__branch)),
            copy(_from=_from, src=self.__prefix,
                 dest=self.__prefix)
        ]
        if self.__environment_variables:
            instructions.append(environment(
                variables=self.__environment_variables))
        if self.__labels:
            instructions.append(label(metadata=self.__labels))
        return '\n'.join(str(x) for x in instructions)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Setup script for hpccm."""

import os

from setuptools import find_packages
from setuptools import setup

from ogscm.version import __version__

here = os.path.abspath(os.path.dirname(__file__))

# Get the long description from the README file
with open(os.path.join(here, 'README.md')) as fp:
    long_description = fp.read()

setup(
    name='ogscm',
    version=__version__,
    description='OGS Container Maker',
    long_description=long_description,
    long_description_content_type='text/markdown',
    maintainer='Lars Bilke',
    maintainer_email='lars.bilke@ufz.de',
    license='BSD 3-Clause',
    url='https://github.com/ufz/ogs-container-maker',
    packages=find_packages(),
    classifiers=[
      "License :: OSI Approved :: BSD License",
      "Programming Language :: Python :: 3",
      "Operating System :: OS Independent",
    ],
    # Make build.py available from the command line as `ogscm`.
    install_requires=['enum34', 'six', 'requests', 'hpccm'],
    # scripts=['ogscm']
    entry_points={
        'console_scripts': [
            'ogscm=ogscm.cli:main']}
)
